function differentials = getEqPoleDiffVsPressure( nc_file_input, epic_vars, vars )

FALSE=0; TRUE=1;
if nargin<3
  error(' Missing input nc_file, epic_vars, var_names.')
end


%
% check for valid input
%
if length( epic_vars ) == 0 
  error([' epic_vars is empty'])
end

if isfield(vars,'name')==FALSE
  error([' Input "vars" must have the format vars(:).name .'])
end

if ischar( nc_file_input )
  nc_file = netcdf( nc_file_input );
else
  nc_file = nc_file_input;
end


%
% get indices cooresponding to variable names
%
for iv=1:length(vars)
  var_found=FALSE;
  for iev=1:length(epic_vars)
    if strcmp(vars(iv).name, epic_vars(iev).name)
      var_found=TRUE;
      vars(iv).id = iev;
    end
  end
  if var_found==FALSE
    error([' Could not find variable "' vars(iv).name '" in epic_vars.'])
  end
end


%
% Remap variable to pressure surfaces
%
for iv=1:length(vars)

  if strfind( vars(iv).name, '3' )   |   strcmp( vars(iv).name, 'theta' )
    [epic_vars, x_data_3] = remapEpicVarToNewSurfaces( epic_vars, nc_file,        ...
                                                       vars(iv).name, 'p3', 'p3', ...
                                                       FALSE, 'all' );
  else
    [epic_vars, x_data_2] = remapEpicVarToNewSurfaces( epic_vars, nc_file,        ...
                                                       vars(iv).name, 'p2', 'p2', ...
                                                       FALSE, 'all' );
  end

end


%
% Average each layer in time
%
for iv=1:length(vars)
  clear t_ave_data
  t_ave_data = epic_vars( vars(iv).id ).data;
  epic_vars( vars(iv).id ).data = [];
  epic_vars( vars(iv).id ).data = mean( t_ave_data, 1 );
end
clear t_ave_data


%
% Determine Jeq
%
for iv=1:length(vars)
  clear lat igt ilt
  lat = getNCvardata( nc_file, epic_vars( vars(iv).id ).dim(3).name );
  igt = find( lat >= 0.0 );
  ilt = find( lat <  0.0 );

  Jeq_p(iv)=-1; Jeq_m(iv)=-1;
  if length(igt)>0
    Jeq_p(iv) = igt(1);
  end
  if length(ilt)>0
    Jeq_m(iv) = ilt(length(ilt));
  end
  if Jeq_p(iv)==-1
    Jeq_p(iv)=Jeq_m(iv);
  end
  if Jeq_m(iv)==-1
    Jeq_m(iv)=Jeq_p(iv);
  end
  if Jeq_p==-1 & Jeq_m==-1
    error(' could not find Jeq_m or Jeq_p')
  end

  if Jeq_p==Jeq_m
    w_p(iv)=0.5;
    w_m(iv)=0.5;
  else
    w_p(iv) = abs(lat(Jeq_p) - 0.0) / abs(lat(Jeq_p)-lat(Jeq_m));
    w_m(iv) = abs(lat(Jeq_m) - 0.0) / abs(lat(Jeq_p)-lat(Jeq_m));
  end
end


% 
% Get Equator minus Pole Differential
%
for iv=1:length(vars)
  clear data eq_m_pole
  data = epic_vars( vars(iv).id ).data;
  N = size(data);
  for K=1:N(2)
    for I=1:N(4)
      equator = w_p(iv)*data(1,K,Jeq_p,I)  +  w_m(iv)*data(1,K,Jeq_m,I);
      eq_m_pole(I) = 0.5*abs( data(1,K,N(3),I) + data(1,K,1,I) - 2.0*equator );
    end
    mean_eq_m_pole(iv,K) = mean( eq_m_pole );
  end
end


%
% Prepare data for output
%
differentials.y_data = mean_eq_m_pole;
if length(who('x_data_2'))
  differentials.x_data_2 = x_data_2;
end
if length(who('x_data_3'))
  differentials.x_data_3 = x_data_3;
end


if ischar( nc_file_input )
  close(nc_file)
end

